﻿using System;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using System.Threading.Tasks;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;
using Windows.UI.Xaml.Controls;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace I2cDisplay
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private struct TempHum
        {
            public double Temp;
            public double Hum;
        }
        public MainPage()
        {
            this.InitializeComponent();
            InitDevice();
        }

        private I2cDevice _tempSensor;
        private I2cDevice _display;
        private Timer _timer;
        private async void InitDevice()
        {
            _tempSensor = await InitI2cDevice(0x40);
            _tempSensor.Write(new byte[] { 0x02, 0x10, 0x00 });

            _display = await InitI2cDevice(0x3C);
            // 画面クリア
            _display.Write(new byte[] { 0x00, 0x01 });
            // カーソルを左上へ
            _display.Write(new byte[] { 0x00, 0x02 });
            // 表示設定
            _display.Write(new byte[] { 0x00, 0x0C });

            _timer = new Timer(MainLoop, null, 0, Timeout.Infinite);
        }

        private async void MainLoop(object state)
        {
            var temp = await GetTemp();
            Write(0, string.Format("Tmp:{0:0.00}C    ", temp.Temp));
            Write(1, string.Format("Hum:{0:0.00}%    ", temp.Hum));
            _timer.Change(1000, Timeout.Infinite);
        }

        private void Write(int v1, string v2)
        {
            if (v1 == 0)
            {
                _display.Write(new byte[] { 0x00, 0x80 });
            }
            else
            {
                _display.Write(new byte[] { 0x00, 0xa0 });

            }
            _display.Write(new byte[] { 0x40 }.Concat(v2.Select(_ => (byte)_)).ToArray());
        }

        private async Task<TempHum> GetTemp()
        {
            // 測定開始
            _tempSensor.Write(new byte[] { 0x00 });
            // センサーの測定待ち。測定にに13[ms]かかるので余裕を持って100[ms]待つ。
            await Task.Delay(100);
            var buf = new byte[4];
            _tempSensor.Read(buf);
            // 仕様書通りに変換
            double temp = (buf[0] * 256.0 + buf[1]) / 65536 * 165.0 - 40.0;
            double hum = (buf[0] * 256.0 + buf[1]) / 65536 * 100.0;
            return new TempHum() { Temp = temp, Hum = hum };
        }

        private static async Task<I2cDevice> InitI2cDevice(byte address)
        {
            var aqs = I2cDevice.GetDeviceSelector("I2C1");
            var device_information_collection = await DeviceInformation.FindAllAsync(aqs);
            var deviceId = device_information_collection[0].Id;
            var connectionSetting = new I2cConnectionSettings(address);
            connectionSetting.BusSpeed = I2cBusSpeed.StandardMode;
            connectionSetting.SharingMode = I2cSharingMode.Shared;

            return await I2cDevice.FromIdAsync(deviceId, connectionSetting);
        }
    }
}
